#ifndef __analysis_h__
#define __analysis_h__
using namespace std;
#ifndef LOWEST_MIDI
#define LOWEST_MIDI 5
#endif
#include <string>
#include <vector>
//Analysis.java
//Perform frequency analysis on digitized sound samples using autocorrelation
//
//(c) 2005 David Cuny
//http://tunafish.sourceforge.net
//Released under Qt Public License
class WaveHandler;
// FIXME: silence minimum width should depend on sample rate
#include "WaveSegment.h"
#include "WaveHandler.h"
class WaveAnalysis {
			
	// lowest possible midi frequency

	
	float sampleRate;
	int windowSize;
	float* samples;
	int sampleslen;



	int zero;// = 0;
	int zeroCutoff;// = 0;
	int silenceMinWidth;// = 200;
	WaveHandler* wavehandler;

	
	// restricted pitches and pitch names
	bool* scalePitch;// = new bool[12];
	static const int C_NOTE = 0;
	
	static const int C_SHARP_NOTE = 1;
	static const int D_FLAT_NOTE = 1;

	static const int D_NOTE = 2;
	
	static const int D_SHARP_NOTE = 3;
	static const int E_FLAT_NOTE = 3;
	
	static const int E_NOTE = 4;

	static const int E_SHARP_NOTE = 5;
	static const int F_FLAT_NOTE = 4;
	
	static const int F_NOTE = 5;
	
	static const int F_SHARP_NOTE = 6;
	static const int G_FLAT_NOTE = 6;
	
	static const int G_NOTE = 7;
	
	static const int G_SHARP_NOTE = 8;
	static const int A__FLAT_NOTE = 9;
	
	static const int A_NOTE = 9;
	
	static const int A_SHARP_NOTE = 10;
	static const int B_FLAT_NOTE = 10;
	
	static const int B_NOTE = 11;
	
	static const int B_SHARP_NOTE = 0;
	static const int C_FLAT_NOTE = 11;

public:	
	int* waveWidth;
	vector<WaveSegment*>* segments; 	
	double myabs(double x);
	bool needsResampling;// = true;
	double* midiValue;
	double* correctedMidiValue;

	float* frequency;
	int maxAmplitude;// = 0;

	WaveAnalysis( float* samples, int sampleslen, int sampleRate, int windowSize ,WaveHandler* handler);
	~WaveAnalysis();
	void setScale(const char* keyName );
	void findMaxAmplitude();
	double closestMatch( double midiPitch);
	void markFrequency( int start, int end );	
	bool near( float error, float a, float b );

	

	/**
	 * This compares the current pitch (represented as a MIDI number) to the target
	 * pitch (supplied by the method <b>closestMatch</b>. If they differ, the pitch is
	 * nudged in the direction of the target pitch by the <b>correctionAmount</b>. This
	 * is done using MIDI pitches instead of actual frequencies, so that the response to
	 * the pitch correction occurs in linear space. Otherwise, the pitch correction would
	 * be faster for higher pitches, since the distance between pitches decreases as the
	 * pitch frequency increases.
	 * @param correctionAmount
	 */
	void calculateCorrectPitch();
	void findSilence();
	int autocorrelate( int at, int localWindowSize );
	void findFrequencies();
	void findSegments();
	
	static double frequencyToMidi( float f );
	static double midiToFrequency( double midi );
	
};

#endif
