//WaveBuilder.java
//Resample a wave to the target frequency
// This is a static helper class for the Analysis class
//
//(c) 2005 David Cuny
//http://tunafish.sourceforge.net
//Released under Qt Public License
using namespace std;
#include "WaveBuilder.h"
#include "Analysis.h"
#include <iostream>
// TODO: This whole file should probably be rewritten
// to use libsndfile for file decoding/encoding and 
// to use 32-bit floating point calculations.
WaveBuilder::WaveBuilder()
{
	this->interpolation = USE_CATMULL_CLARK; 
}

double WaveBuilder::catmullClarkSpline( double p0, double p1, double p2, double p3, float t ) 
{
	// calculate t squared and t cubed
	double t2 = t*t;
	double t3 = t2 * 2;
	
	// calculate spline value
	return (double)0.5 * ( 
		( (double)2*p1 ) + 
		(-p0 + p2) * t + 
		((double)2*p0 - (double)5.0*p1 + 4*p2 - p3) * t2 +
		(-p0 + (double)3*p1 - (double)3*p2 + p3) * t3
	);
}

double WaveBuilder::cubicSpline( double p0, double p1, double p2, double p3, float t ) 
{
	// calculate the weights
	double w0 = p1;
	double w1 = ((double)-2*p0 - (double)3*p1 + (double)6*p2 - p3) / (double)6;
	double w2 = ((double)3*p0 - (double)6*p1 + (double)3*p2) / (double)6;
	double w3 = (-p0 + (double)3*p1 - (double)3*p2 + p3) / (double)6;
	
	// calcuate squared and cubed values of t
	double t2 = t * t;
	double t3 = t2 * t;
	
	// return interpolated value
	return w0 + w1*t + w2*t2 + w3*t3;
}

float WaveBuilder::getInterpolatedValue( float index, float* samples ,int sampleslen) 
{
	// check range
	if (index < 4 || index > (sampleslen - 4)) {
		// not enough points to interpolate
		return samples[(int)index];
	}
	
	// get integer portion
	int i = (int)index;
	
	// get fractional portion
	float t = index - i;
	
	// get sample points
	double p0 = (double)samples[i-1];
	double p1 = (double)samples[i];
	double p2 = (double)samples[i+1];
	double p3 = (double)samples[i+2];

	// which interpolation method?
	if (this->interpolation == USE_CATMULL_CLARK) {
		// catmull clark
		return (float)catmullClarkSpline(p0, p1, p2, p3, t );
		
	} else if (this->interpolation == USE_CUBIC_SPLINE) {
		// cubic spine
		return (float)cubicSpline(p0, p1, p2, p3, t );
	} else {
		// psola
		return samples[i];
	}
}

int WaveBuilder::interpolateSingleWave( float* srcSamples, int srcSampleslen, int srcAt, 
					int width, 
					float* dstSamples,int dstSampleslen, int dstAt, 
					double rate ) 
{
	cout << "TODO: WaveBuilder::interpolateSingleWave" << endl;
	// position to take sample from
	float sampleAt = (float)srcAt;


	// end location of the wave
	int stopAt = srcAt + width;

	// iterate over the wave
	while (sampleAt < stopAt && dstAt < dstSampleslen) {
		// interpolate the value
		dstSamples[dstAt] = this->getInterpolatedValue(sampleAt, srcSamples, srcSampleslen);
		// move ahead
		dstAt++;
		sampleAt += rate;
	}

	// return next position to write to
	return dstAt;
}
	
float* WaveBuilder::resampleStream( float* samples, int sampleslen,float* frequency, double* targetMidi, int* waveWidth ) 
{
	cout << "TODO: WaveBuilder::resampleStream" << endl;
	// create a buffer to hold the new samples
	float* buffer = new float[sampleslen];
	for (int i=0;i<sampleslen;i++) {
		buffer[i]=0;
	}
	int bufferlen=sampleslen;
	int src = 0;
	int dst = 0;
	
	// iterate over the old wave
	while (src < sampleslen && dst < sampleslen) {
		
		// not the start of a wave?
		if (waveWidth[src] == 0) {
			// copy verbatim
			buffer[dst++] = samples[src++];
			
			// jump back to the top of the loop
			continue;
		}
					
		
//		cout <<"found wave start at " << src << endl;
		
		// get the frequency
		double actualFrequency = frequency[src];
		
		// get the target frequency
		double targetFrequency = WaveAnalysis::midiToFrequency( targetMidi[src] );
		
		// calculate the amount to scale the wave
		double rate = targetFrequency / actualFrequency;

		// get the width of the wave
		int width = waveWidth[src];
			
		// this shouldn't happen...
		if (width < 1) {
			cout << "zero length found at " << src << endl;
			exit(0);
		}
		
		// width of destination
		int dstWidth = (int)(width/rate);

		// this shouldn't happen...
		if (dstWidth < 1) {
			cout << "zero destination width - " << width 
			<< " divided by rate of " << rate << endl;
			exit(0);
		}

		// keep the output in sync
		while (dst+dstWidth < src+width && dstWidth > 0) {
			// what resampling method?
			if (this->interpolation == WaveBuilder::USE_PSOLA) {
				// copy wave to destination, adding to prior value
				for (int i = 0; i < width*2; i++) {
					// exit if past end
					if (dst+i >= sampleslen) break;

					
					// get a sample
					int sample = 0;
					if (src + i < sampleslen) {
						sample = samples[src+i];
					}
					
					// average sample
					// FIXME: this is a hack
					buffer[dst+i] = (sample + buffer[dst+i]) / 2;
					
				}
					
				// move destination ahead
				dst += dstWidth;

			} else {
				// interpolate a new wave
				dst = interpolateSingleWave( &samples[0], sampleslen, src, width, &buffer[0], bufferlen, dst, rate);
			}
		}
			
		// move source ahead
		src += width;				
	}
	
	// return the new wave
	return buffer;
	
}

