//  WaveFile.java
// Load, display and play wave files
//
// (c) 2005 David Cuny
// http://tunafish.sourceforge.net
// Released under Qt Public License
#include "WaveHandler.h"
#include "convertlib.h"
#include "utils.h"
#include "Analysis.h"
#include <FL/fl_draw.H>
#include <vector>
#include "ColorScheme.h"
#include "FastFourierTransform.h"
#include <sndfile.h> // TODO: replace with runtime loader.
#define BUFFER_LEN 1024
#define MAX_CHANNELS 2

#define WAVEHAN_DEBUG 1

void WaveHandler::init()
{
#if (WAVEHAN_DEBUG==1)
	cout << "WaveHandler::init()" << endl;
#endif
	this->zoom = 512;
	this->CORRECTION_RATE = .0003;
//	this->rawBytes = NULL;	
	this->originalSamples = NULL;
	this->modifiedSamples = NULL;
	this->originalSampleslen=0;
	this->wavePanel=NULL;
	this->waveBuilder=new WaveBuilder;
	this->sfinfoin=new SF_INFO;
	this->sfinfoout=new SF_INFO;

	this->needsResampling = false;
	
	// playback volume
	this->playBackVolume = 5;
	
	// selection
	this->selectionStart = -1;
	this->selectionEnd = -1;
	

	// allow stopping of wave playback
	this->stopPlayingWave = false;
		
	// name of file
	this->fileName = new string("");
	this->file = NULL;

	// output data line
	sourceDataLine = NULL;
	
	// audio format
	sourceAudioFormat = NULL;
	
	// stream
	sourceAudioInputStream = NULL;

	analyzeWave=NULL;//new WaveAnalysis();
}

WaveHandler::WaveHandler() throw()
{
#if (WAVEHAN_DEBUG==1)
	cout << "WaveHandler::WaveHandler()" << endl;
#endif
	this->init();
}

void WaveHandler::fileOpen() 
{
#if (WAVEHAN_DEBUG==1)
	cout << "WaveHandler::fileOpen()" << endl;
#endif
	// autosave?
	if (cancelledAutoSave()) {
		return;
	}
	// open file dialog

	string* wavedir=utils::getlastdir("wavedir");

	Fl_Native_File_Chooser chooser;
	chooser.directory(wavedir->c_str());
	delete wavedir;

	chooser.title("Select a file to load\0");
	chooser.type(Fl_Native_File_Chooser::BROWSE_FILE);
	chooser.filter("Audio files\t*.{wav}\0");
//chooser.preview(0);
	switch (chooser.show()) 
	{
		case -1: break; //error
		case 1: break; //cancel
		default:
			if (chooser.filename()) {
				string* cfilename=new string(chooser.filename());
	
				string* fpath=new string("");
				*fpath+=cfilename->substr(0,strlen(cfilename->c_str())-strlen(fl_filename_name(cfilename->c_str())));
				utils::setlastdir("wavedir",fpath->c_str());
				this->loadWave(cfilename);	
				delete cfilename;
				delete fpath;
		}
		break;
	}

}	

void WaveHandler::fileSave() {
#if (WAVEHAN_DEBUG==1)
	cout << "WaveHandler::fileSave()" << endl;
#endif
	if (this->file == NULL) {
		// use save as instead
		fileSaveAs();
	} else {
		// write out the wave
		cout << "TODO: file->getpath();" << endl;
//		this->writeFile(this->file->getPath() );
	}
}

int WaveHandler::confirm(const char* question)
{
	int result=fl_choice(question,"No","Yes",NULL);
	if (result==1) {
		return true;
	}
	return false;
}

void WaveHandler::fileSaveAs() {
#if (WAVEHAN_DEBUG==1)
	cout << "WaveHandler::fileSaveAs()" << endl;
#endif
	cout << "TODO: fileSaveAs" << endl;
	string* wavedir=new string("");
	*wavedir+=*utils::getlastdir("wavedir");

	Fl_Native_File_Chooser chooser;
	chooser.filter("Audio files\t*.{wav}\0");
	chooser.title("Save audio file as");
	chooser.directory(wavedir->c_str());
	chooser.options(Fl_Native_File_Chooser::NEW_FOLDER);
	chooser.type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);

	switch (chooser.show()) {
		case -1: break; //error
		case 1: break; //cancel
		default:
			// save header to chooser.filename()
			bool bFileexists=utils::fileExists(chooser.filename());
			if (bFileexists) {
				bool choice=confirm(
					"A file with this name already exists. Do you wish to overwrite it?"
				);			
				if (!(choice)) return;
			}
	
			string* strfile=new string(chooser.filename());
		
			string* fpath=new string("");
			*fpath+=strfile->substr(0,strlen(strfile->c_str())-strlen(fl_filename_name(strfile->c_str())));
			utils::setlastdir("wavedir",fpath->c_str());

			writeFile(strfile);
			break;
	}

}

// ensure file name ends with given extention
string* WaveHandler::addFileExtention(string* fileName, string* extention) 
{
#if (WAVEHAN_DEBUG==1)
	cout << "WaveHandler::addFileExtention()" << endl;
#endif
	string* lowerFileName = new string(*fileName);
	Convert::makelower(lowerFileName);
	cout << "TODO: addFileExtention" << endl;
/*
	if (!(lowerFileName->endsWith(extention))) {
		return fileName + extention;
	}*/
	return fileName;
}

	
// if returns false, user cancelled autosave action
bool WaveHandler::cancelledAutoSave() {
#if (WAVEHAN_DEBUG==1)
	cout << "WaveHandler::cancelledAutoSave()" << endl;
#endif
	// no work to save?
	if (this->file == NULL) {
		return false;
	}

	// ask user if they want to save their current work
	cout << "TODO: cancelledautosave confirmdialog" << endl;
/*
	int result = JOptionPane.showConfirmDialog(NULL,
			"Save Current File First?", "Save File?",
			JOptionPane.YES_NO_CANCEL_OPTION);
	if (result == JOptionPane.CANCEL_OPTION) {
		// cancelled
		return true;

	} else if (result == JOptionPane.NO_OPTION) {
		// didn't save, but didn't cancel
		return false;
	}

	// make sure it ends with the proper extention
	String openFileName = this->file.getPath();
	openFileName = addFileExtention(openFileName, ".wav");

	// write the new wave file out
	writeFile( openFileName ); 
*/
	return false;

}

void WaveHandler::startSelection( int at ) 
{
#if (WAVEHAN_DEBUG==1)
	cout << "WaveHandler::startSelection()" << endl;
#endif
	cout << "startSelection at " << at ;
	this->selectionStart = at;
	this->selectionEnd = -1;
}
	
void WaveHandler::extendSelection( int at ) {
#if (WAVEHAN_DEBUG==1)
	cout << "WaveHandler::extendSelection()" << endl;
#endif
	if (at < this->selectionStart) {
		cout << "extendSelection at " << at << endl;
		this->selectionEnd = this->selectionStart;
		this->selectionStart = at;
	} else {
		cout << "extendSelection end at " << at << endl; 
		this->selectionEnd = at;
	}
}

string* WaveHandler::midiNoteName( int i ) 
{
	string *result;
	int octave = (i / 12) - 2;
	switch (i % 12) 
	{
		case 0: 
			result=new string("C");
			break;
		case 1:
			result=new string("C#");
			break;
		case 2:
			result=new string("D");
			break;
		case 3:
			result=new string("D#");
			break;
		case 4:
			result=new string("E");
			break;
		case 5:
			result=new string("F");
			break;
		case 6:
			result=new string("F#");
			break;
		case 7:
			result=new string("G");
			break;
		case 8:
			result=new string("G#");
			break;
		case 9:
			result=new string("A");
			break;
		case 10:
			result=new string("A#");
			break;
		case 11:
			result=new string("B");
			break;
		default:
			result=new string("?");
			return result;
	}
	string* oct=Convert::int32tostr(octave);
	*result+=*oct;
	delete oct;
	return result;
}

int WaveHandler::midiToPos( double midi, int height ) 
{
	return height - (int)((midi-40)*SEGMENT_HEIGHT);
}
	
// FIXME: needs to add x offset
WaveSegment* WaveHandler::hitSegment( int x, int y, int xoff,int yoff,int height ) 
{
#if (WAVEHAN_DEBUG==1)
	cout << "WaveHandler::hitSegment()" << endl;
#endif
	// no waveform?
	if (this->analyzeWave==NULL) 
	{
		return NULL;
	}
	if (this->sfinfoin->frames == 0) 
	{
		return NULL;
	}
	cout << "TODO: WaveHandler::hitSegment" << endl;
	// get the wave segments
	vector<WaveSegment*>* segments;
	
	segments = this->analyzeWave->segments;
	
	// iterate over them
	for (int i = 0; i < segments->size(); i++) 
	{
		// get a segment
		WaveSegment* segment = (*segments)[i];
		
		// calculate the position on the screen
		int xStart = (int)(segment->startSample / this->zoom);
		int yStart = midiToPos( segment->midiNumber, height );
		
		// render the segment
		segment->calcScreenPosition( xStart, yStart, this->zoom ,this);
/*
		fl_color(FL_BLUE);
		fl_rect( xoff+MARGIN+segment->x1, yoff+segment->y1, xoff+1+segment->x2 - segment->x1,1+ segment->y2 - segment->y1);
*/
		// inside segment?
		if (
			((xoff+MARGIN+(segment->x1)) < x) 
			&& 
			((xoff+MARGIN+(segment->x2)) > x)
		) {
			if (
				((yoff+(segment->y1)) < y) 
				&& 
				((yoff+(segment->y2)) > y) 
			) {
				return segment;
			}
		}
	}
	// no segement hit
	return NULL;
	
}

bool WaveHandler::isBlackKey( int midiNote ) 
{
	int scaleNumber = (midiNote+11) % 12;

	switch (scaleNumber) 
	{
		case 1:
		case 3:
		case 6:
		case 8:
		case 10:
			// black key
			return true;
		default:
			// white key
			return false;
	}
}
	
void WaveHandler::render_pitchscale(int xpos,int ypos,int width,int height, int waveOffset)
{
	// draw the lines for the midi pitch scale
	for (int i = 127; i >= 0; i--) {
		int y = this->midiToPos( i-.5, height );
		if (y<0) continue;
		y+=ypos;
		
		if (isBlackKey(i)) 
		{
			fl_color(SCALE_BACKGROUND);
		} 
		else 
		{
			fl_color(NON_SCALE_BACKGROUND);
		}
		fl_rectf( 0, y, width, y+SEGMENT_HEIGHT );
		
		// draw a dividing line
		fl_color(SCALE_DIVIDING_LINE);
		fl_line( 0, y, width, y );
		
		// draw the note name (if it fits)
		if (y >height+ypos) continue;
		if (y>=ypos+8) {
			fl_color(PITCH_NAME_FOREGROUND);
			string* NoteName=midiNoteName(i);	
			int font=fl_font();
			int fontsize=12;
			fl_font(font,fontsize);
			fl_draw( NoteName->c_str(), 5, y );
			delete NoteName;
		}
	}

}	

void WaveHandler::drawPolyline(int* x,int* y,int size)
{
	if (size<=0)
	{
		return;
	}
	fl_line(x[0],y[0],x[0],y[0]);
	cout << "polyline" << size << endl;
	if (size==1)
	{
		return;
	}
	cout <<"("<<x[0] << ","<< y[0]<<")";
	for (int i=1;i<size;i++)
	{
		
//		if (y[i]<0) { y[i]=y[i-1]; }
		//if ((x[i]==0) && (y[i]==0)) { y[i]=y[i-1]; x[i]=x[i-1]; }
		cout <<"("<<x[i] << ","<< y[i]<<")";
		fl_line(x[i-1],y[i-1],x[i],y[i]);
	}
	cout << endl;
}

void WaveHandler::render_selection (int xpos,int ypos,int width,int height,int waveOffset)
{
	// is there a selection?
	if (selectionStart == -1) {
		return;
	}

	fl_color(SELECTION);
	cout << "selectionStart=" << selectionStart << "off="<<waveOffset <<"zoom=" <<zoom << endl;
	int start = (int)((selectionStart-MARGIN) / zoom) - ( waveOffset/zoom);
	if (selectionEnd == -1) {
		// only draw selection line
		fl_rectf( MARGIN+start+xpos, ypos, 1, height ); // TODO: start+xpos-waveoffset?
	} else {				
		int end = (int)(selectionEnd / zoom) - (waveOffset/zoom);
		// draw selection area
		fl_rectf( MARGIN+start+xpos, ypos, 1+end-start, height );			

	}

	// no waveform?
	if (this->sfinfoin->frames == 0) {
		return;
	}
	
	int sampleCount = 1024;
	
	// select a window of 512 samples
	float* sourceSamples = (float*)malloc(sampleCount*sizeof(float));
	if (sourceSamples==NULL)
	{
		return;
	}

	// copy the samples into the array
	for (int i = 0; i < sampleCount; i++) {
		sourceSamples[i] = this->originalSamples[selectionStart+i];
	}
	
	FastFourierTransform* fft=new FastFourierTransform();
	// analyze them
	float* mag = fft->fftMag(sourceSamples,sampleCount);
	
	// result is half the size of the sample count
	sampleCount /= 2;
	
	// build a polyline
	int maxAt = 0;
	int* magX = (int*)malloc(sampleCount*sizeof(int));
	int* magY = (int*)malloc(sampleCount*sizeof(int));
	for (int i = 0; i < sampleCount; i++) {
		// copy to polyline
		magX[i] = i + 20;
		magY[i] = (int)(mag[i]);//*0.05);
		
		// largest so far?
		if (mag[maxAt] < mag[i]) {
			// save value
			maxAt = i;
		}
	}

	free(mag);

	
	// render the plot
	fl_color(FL_ORANGE);
	this->drawPolyline( magX, magY, sampleCount );
	free (magX);
	free (magY);	


	// calculate frequency
    	float f = 0.5f * ((float)this->sfinfoin->samplerate * (float)maxAt) / (float)sampleCount;
	
	// display the value
	fl_color(FL_BLACK);
	string* todraw=new string("Frequency=");
	string* freq=Convert::int2str(f);
	*todraw+=*freq;
	*todraw+="Hz";
	delete freq;	
			int font=fl_font();
			int fontsize=12;
			fl_font(font,fontsize);
	fl_draw(todraw->c_str(), 10, 10+ypos);	
	delete todraw;
	delete fft;
}

void WaveHandler::render_segments( int xpos,int ypos,int width,int height, int waveOffset)
{
	// get the wave segments
	if (this->analyzeWave==NULL)
	{
		return;
	}
	if (this->analyzeWave->segments==NULL)
	{
		return;
	}
	vector<WaveSegment*>* segments = this->analyzeWave->segments;
	if (segments==NULL)
	{
		return;
	}
	int segsize=segments->size();
	// iterate over them
//	cout << "render " << segsize << " segments" << endl;
	for (int i = 0; i < segsize; i++) {
		// get a segment
		WaveSegment* segment = (*segments)[i];
		
		// calculate start position on the screen
		int xStart = (int)(segment->startSample / zoom);
		int yStart = midiToPos( segment->midiNumber, height );
		
		// render the segment
		segment->render(xStart+xpos+MARGIN, yStart+ypos, zoom, this,this->modifiedSamples,this->sfinfoin->frames,waveOffset);
		// TODO: should samplecount be blocksize or wave size?
	}
}

void WaveHandler::render_waves( int xpos,int ypos,int width,int height, int waveOffset)
{
	// get the wave segments
	if (this->analyzeWave==NULL)
	{
		return;
	}
	if (this->analyzeWave->midiValue==NULL)
	{
		return;
	}
	// center of wave
	int center = height - WAVE_HEIGHT/2;

	// prepare polyline
	int* xPos = (int*)malloc(width*sizeof(int));
	int* yPos = (int*)malloc(width*sizeof(int));

	// cache points for frequency and corrected frequency
	int* xFreqPos = (int*)malloc(width*sizeof(int));
	int* yFreqPos = (int*)malloc(width*sizeof(int));

	int* yFixedFreqPos = (int*)malloc(width*sizeof(int));


	// cache points for corrected frequency
	int* newFreqPos = (int*)malloc(width*sizeof(int));

	for (int i=0;i<width;i++) {
		xPos[i]=0;
		yPos[i]=0;
		xFreqPos[i]=0;
		yFreqPos[i]=0;
///		xFixedFreqPos[i]=0;
		yFixedFreqPos[i]=0;
	}
	
	// draw the wave
	fl_color(WAVE_COLOR);
	float at = waveOffset;
	int zero = this->modifiedSamples[0];
	int halfHeight = WAVE_HEIGHT / 2;
	int freqWidth = 0;

	for (int x = 0; x < width; x++) {
		// increment position
		at += zoom;
		int i = (int)at;
		
		// past range?
		if (i >= this->sfinfoin->frames) 
		{
			break;
		}

		// get sample value for frame
		int value = 0;
		
		// invert the value for display
		value = -this->modifiedSamples[i] - zero;
		 
		// fit sample to the window
		value = center + (int)(halfHeight *((float)value/(float)this->analyzeWave->maxAmplitude));			
					
		// store for polyline
		xPos[x] = x+xpos+MARGIN;
		yPos[x] = value+ypos;
		cout << "freqWidth=" << freqWidth << endl;				
		// store frequency
		bool breakwave=false;
		if (i < this->sfinfoin->frames)
		{
			// x position
			xFreqPos[freqWidth] = x+MARGIN;

			// get analyzed pitch

			double midi = this->analyzeWave->midiValue[i];
			if ((midi<0)||(midi>127)) breakwave=true;
			int mpos=midiToPos( midi, height ); 
			if (mpos<=0) breakwave=true;
			yFreqPos[freqWidth] = ypos+mpos;
			
			// get corrected pitch
			midi = this->analyzeWave->correctedMidiValue[i];
			if ((midi<0)||(midi>127)) breakwave=true;
			mpos=midiToPos( midi, height ); 
			if (mpos<=0) breakwave=true;
			
			yFixedFreqPos[freqWidth] = ypos+ mpos;
			if (
				(yFreqPos[freqWidth]<0)
				|| (yFixedFreqPos[freqWidth]<0)			
			) {
				breakwave=true;
			}
			cout << "yFreqPos[freqWidth]=" << yFreqPos[freqWidth] << endl;				
			cout << "yFixedFreqPos[freqWidth]=" << yFixedFreqPos[freqWidth] << endl;				
			cout << "midivalue [" << i << "] = " << this->analyzeWave->midiValue[i] << endl;
			
			freqWidth ++;
			
		}

		if (this->analyzeWave->frequency[i]<20) breakwave=true;

		// draw the accumulated wave?
		// FIXME: bottom test is a hack...

		if (	 breakwave || (x == width-1)) 
		{
			// draw the frequency?
			if (freqWidth > 0) {
				
				// ignore this point
				freqWidth --;

				if (freqWidth > 0) {
					// frequency
					fl_color(FL_RED);
					this->drawPolyline( xFreqPos, yFreqPos, freqWidth );
			
					// corrected frequency
					fl_color(FL_GREEN);
					this->drawPolyline( xFreqPos, yFixedFreqPos, freqWidth );
				
					// reset the wave
					freqWidth = 0;
				}

			}
		}
	}
	
	// draw the wave
//	fl_color(OUTER_LINE);
//	this->drawPolyline( xPos, yPos, width );				
}
void WaveHandler::render( int xpos,int ypos,int width,int height, int waveOffset)
{

	render_pitchscale(xpos,ypos,width,height,waveOffset);				
	render_selection(xpos,ypos,width,height,waveOffset);					
	//g.drawString( "Zoom = " + zoom, 10, height-30);
	//if (selectionStart != -1) {
//		//g.drawString( "Point " + selectionEnd + "=" + this->analyzeWave.samples[selectionEnd], 10, 30);
	//}

	render_segments(xpos,ypos,width,height,waveOffset);					

	// render_fileinfo():
	//fl_color(FL_BLACK);
//	fl_draw("File=",10,ypos+(height-10));
	//cout << "bits per sample =" + this->bitsPerSample);
	//cout << "bytes per sample =" + this->bytesPerSample);
	//cout << "channels =" + this->channels);
	//cout << "signed =" + this->signed);
	render_waves(xpos,ypos,width,height,waveOffset);			

	
}
	
fstream* WaveHandler::askUserForFile(string* fileName) 
{	
	cout << "TODO: WaveHandler::askUserForFile" << endl;	
	/*
	// prompt user
	int result = JOptionPane.showConfirmDialog(NULL,
			"Can't find the file '" + fileName
					+ "'\nDo you want to find it manually?",
			"Unable to find .wav file", JOptionPane.YES_NO_OPTION);

	if (result == JOptionPane.YES_OPTION) {					
		// open file dialog
		JFileChooser fc = Utilities.fileChooser(NULL);

		// limit to .wav files
		fc.addChoosableFileFilter(new WavFilter());
		
		// open the dialog			
		int returnVal = Utilities.showOpenDialog(fc);
		
		// check results
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			return fc.getSelectedFile();
		}
	}
	*/
	// no file
	return NULL;

}
	
// write a waveform to a file
void WaveHandler::writeFile(string* fileName) 
{
	cout << "TODO:: WaveHandler::writeFile" << endl;
	this->resampleWaveIfRequired() ;
	SNDFILE* filehandle;
	sfinfoout->frames=0;
	sfinfoout->samplerate=sfinfoin->samplerate;
	sfinfoout->channels=sfinfoin->channels;
	sfinfoout->format=sfinfoin->format;
	sfinfoout->sections=1;
	sfinfoout->seekable=0;
	filehandle=sf_open(fileName->c_str(),SFM_WRITE,this->sfinfoout);
	if (!filehandle)
	{
		cout << "Error opening file." << endl;
		int x=sf_format_check(sfinfoout);
		cout << sf_strerror(filehandle) << endl;
		return;
	}
	int x=sf_format_check(sfinfoout);
	cout << sf_strerror << endl;
	unsigned long writecount=0;
	while (writecount<(sfinfoin->frames))
	{
		int framestowrite=sfinfoin->frames-writecount;
		if (framestowrite>BUFFER_LEN) {
			framestowrite=BUFFER_LEN;
		}
		float* ptr=this->modifiedSamples;
		ptr+=writecount;	
		sf_write_float(filehandle,ptr,framestowrite);
		writecount+=framestowrite;
		cout << "Writecount=" << writecount << endl;
	}
	sf_write_sync(filehandle);
	sf_close(filehandle);
	return;



/*
	try {
	
		// create a file to write to
		File* outFile = new File( fileName );
	
		// create an input stream with the data
		ByteArrayInputStream* inputStream = new ByteArrayInputStream(this->rawBytes);
	
		long frameSize = this->byteCount / sourceAudioFormat.getFrameSize();
	
		// convert it to an audio stream		
		AudioInputStream outputStream = new AudioInputStream(
			inputStream, sourceAudioFormat, frameSize );
			
		// write out to a file
		int writtenBytes = AudioSystem.write(outputStream, AudioFileFormat.Type.WAVE, outFile);
		
	} catch (Exception e) {
		Utilities.errMessage( "Error writing file '" + fileName + "'" );
	}*/
}
	
void WaveHandler::readWaveStream( fstream* file ) 
{
	cout << "TODO: WaveHandler::readWaveStream" << endl;

}
	

// load a waveform from a file
void WaveHandler::loadWave(string* fileName) 
{	
	SNDFILE* filehandle;
	filehandle=sf_open(fileName->c_str(),SFM_READ,this->sfinfoin);
	if (!filehandle) 
	{
		this->sfinfoin->frames=0;
		//bytecount=0;
		this->file=NULL;
		this->fileName=NULL;
		return;
	}

	if (originalSamples!=NULL)
	{
		free(originalSamples);
		originalSamples=NULL;
	}
	if (modifiedSamples!=NULL)
	{
		free(modifiedSamples);
		modifiedSamples=NULL;
	}
	
	// number of channels
	this->channels = this->sfinfoin->channels;
  	__uint32 sampleRate=this->sfinfoin->samplerate;
	// allocate room for the samples
	this->originalSamples = (float*)malloc(sizeof(float)*(BUFFER_LEN+((this->sfinfoin->channels)*(this->sfinfoin->frames))));
	if (this->originalSamples==NULL)
	{
		sf_close(filehandle);
		this->sfinfoin->frames=0;
		return;
	}

	this->modifiedSamples = (float*)malloc(sizeof(float)*(BUFFER_LEN+((this->sfinfoin->channels)*(this->sfinfoin->frames))));
	if (this->modifiedSamples==NULL)
	{
		sf_close(filehandle);
		this->sfinfoin->frames=0;
		return;
	}

	long long totreadcount=0;
	__uint32 readcount=0;
	do
	{
		readcount = sf_read_float (filehandle, &this->originalSamples[totreadcount*channels], BUFFER_LEN);
		totreadcount+=readcount;
		//cout << "read a total of " << totreadcount << "samples into buffer of size "<< this->sfinfoin->frames << " frames" << endl;
	} while (readcount!=0);

	sf_close(filehandle);

	long long i;
	this->originalSampleslen=totreadcount*channels;
	for (i = 0; i < (totreadcount*channels); i++) {
		this->modifiedSamples[i] = this->originalSamples[i];
	}	    

	// adjust the scrollbar
	if (this->wavePanel!=NULL)
	{
		this->wavePanel->adjustScrollbar();
	}
			
	// analyze the wave
	if (this->analyzeWave!=NULL)
	{
		delete (this->analyzeWave);
	}
	
	cout << "totreadcount=" << totreadcount << endl;
	this->analyzeWave = new WaveAnalysis(this->originalSamples, totreadcount,sampleRate, 1024,this);
	
	// find the frequencies
	this->analyzeWave->findFrequencies();
	
	// find the individual segments
	this->analyzeWave->findSegments();
	
	// no pitch correction
	this->restoreOriginal(); 
	

}


void WaveHandler::restoreOriginal() 
{

	// set the scale
	this->analyzeWave->setScale( "chromatic" );

	// copy original samples
	for (int i = 0; i < this->originalSampleslen; i++) {
		this->modifiedSamples[i] = this->originalSamples[i];
	}
	
	// set correction rate to zero
	double oldRate = this->CORRECTION_RATE;
	this->CORRECTION_RATE = 0.0;
	this->analyzeWave->calculateCorrectPitch();
	
	// restore correction rate
	this->CORRECTION_RATE = oldRate;
	

}
	
void WaveHandler::resampleWaveIfRequired() 
{
	cout << "TODO: WaveHandler::resampleWaveIfRequired " << endl;
	// skip if no analysis
	if (this->analyzeWave == NULL) return;
	if (this->sfinfoin==NULL) return;
	if ((this->sfinfoin->frames)==0) return;
	// needs resampling?
	if (this->analyzeWave->needsResampling) {		
		
		// resample the wave to the new scale
		this->modifiedSamples = this->waveBuilder->resampleStream( 
			&(*this->originalSamples),
			this->sfinfoin->frames,
			&(*this->analyzeWave->frequency), 
			&(*this->analyzeWave->correctedMidiValue), 
			&(*this->analyzeWave->waveWidth) 
		);
	
		// convert samples back to bytes for playback
		// (Not needed as we work with floats)
		//WaveBuilder.samplesToBytes( this->modifiedSamples, this->rawBytes, this->bytesPerSample );
		
		// clear flag
		this->analyzeWave->needsResampling = false;

	}
}
	
void WaveHandler::setVolume( float newVolume ) 
{
	// this handles events from the volume slider control
	
	// store the value in playbackVolume
	this->playBackVolume = newVolume;
	
	// adjust the volume if a dataline is open
	adjustVolume();
}
		
	
void WaveHandler::adjustVolume() 
{
}

// play the wave file
void WaveHandler::playWave(int startSample, int endSample ) 
{
	// resample the wave, if needed
	this->resampleWaveIfRequired();
	
	// no start sample?
	if (startSample == -1) {
		startSample = 0;
	}
	cout << "TODO: WaveHandler::playWave" << endl;
/*
	// no end frame or past end
	if (endSample == -1 || endSample > originalSamples.length ) {
		endSample = originalSamples.length;
	}

	// play the thread on a background thread
	try {
		// calculate start and end byte sample
		int startByte = startSample * this->bytesPerSample;
		int endByte = endSample * this->bytesPerSample;
		
		// start the thread
		new PlayThread(startByte, endByte, rawBytes).start();
	} catch (Exception e) {
		Utilities.errMessage("Error playing the wave file");
	}*/
}

/*	// play the sound on a background thread
	class PlayThread extends Thread {

		int startByte, endByte;
		byte* samples;

		public PlayThread(int startAtByte, int endAtByte, byte* samples) {
			this->startByte = startAtByte;
			this->endByte = endAtByte;
			this->samples = samples;
			stopPlayingWave = false;
		}

		public final void run() {

			// make sure a waveform is loaded
			if (this->sfinfoin->frames == 0) {
				return;
			}

			// make sure audio device connected
			if (sourceDataLine == NULL) {
				Utilities.errMessage("Audio device failed. Try reloading file");
				return;
			}

			// is it open?
			if (!(sourceDataLine.isOpen())) {
				Utilities.errMessage("Data line is not open");
				sourceDataLine = NULL;
			}
			
			// adjust the dataline volume
			adjustVolume();
			
			// send out 1K at a time
			int bufferSize = (1024 * 4);						

			// calculate buffer
			int startOfBuffer = startByte;

			while (!WaveHandler.stopPlayingWave) {
				// check for buffer overflow
				if ((startOfBuffer + bufferSize) >= endByte) {
					bufferSize = (endByte) - startOfBuffer;
				}

				// start the line
				try {
					sourceDataLine.start();
				} catch (Exception e) {
					// oops
					cout << "Unable to start the line");
				}
				
				// fixme: throws illegal length if too large				
				int nBytesWritten = sourceDataLine.write(samples, startOfBuffer, bufferSize);

				// point to next starting point in the buffer
				startOfBuffer += bufferSize;
				if (startOfBuffer >= endByte) {
					break;
				}

			}
			
			// play out the remainder of the line, but don't close it
			sourceDataLine.stop();
			sourceDataLine.drain();
			sourceDataLine.flush();

		}

		
	}
*/	

