//  WaveFile.java
// Load, display and play wave files
//
// (c) 2005 David Cuny
// http://tunafish.sourceforge.net
// Released under Qt Public License
#ifndef __WAVEHANDLER_H_
#define __WAVEHANDLER_H_
class WaveSegment;
class WaveAnalysis;
class WavePanel;

#include "config.h"
#include <string>
#include <iostream>
#include "WaveSegment.h"
#include "WavePanel.h"
#include "WaveBuilder.h"
#include "Analysis.h"
#include <sndfile.h> // TODO: replace with runtime loader.

class WaveHandler 
{
	

	
	
	// buffers holding the wave data as samples and bytes 
//	byte* rawBytes; //= null; // we don't need this
	float* originalSamples; // float32
	float* modifiedSamples; // float32

	int channels;// = 1; // mono or stereo		
	/* See if I care about these- internally we'll work with
	   signed 32 bit float, and for saving we keep the SFINFO structure. */
	//int bitsPerSample;// = 0; // number of signifcant bits
//	int bytesPerSample;// = 1; // 1 = 8 bit, 2 = 16 bit

//	bool issigned;// = false;	
//	float sampleRate;
	//int byteCount;// = 0;

	bool needsResampling;// = false;
	
	// playback volume
	float playBackVolume;// = 5;
	

	int confirm(const char* question);
	// name of file
	string* fileName;// = new string("");
	fstream* file;// = null; // LIBSNDFILE?

	// output data line for audio (SourceDataLine*)
	void* sourceDataLine;// = null;
	
	// audio format (AudioFormat*)
	void* sourceAudioFormat;// = null;
	
	// stream
	void* sourceAudioInputStream;// = null;
	/*(AudioInputStream*) An audio input stream is an input stream with a specified audio format and length. The length is expressed in sample frames, not bytes. Several methods are provided for reading a certain number of bytes from the stream, or an unspecified number of bytes. The audio input stream keeps track of the last byte that was read. You can skip over an arbitrary number of bytes to get to a later position for reading. An audio input stream may support marks. When you set a mark, the current position is remembered so that you can return to it later. */


	fstream* askUserForFile(string* fileName);
	int originalSampleslen;
private:
	void init();
	WavePanel* wavePanel;
	WaveBuilder* waveBuilder;
	void drawPolyline(int* x,int* y,int size);
	void render_pitchscale(int x,int y,int width,int height, int waveOffset);
	void render_selection(int x,int y,int width,int height, int waveOffset);
	void render_segments(int x,int y,int width,int height, int waveOffset);
	void render_waves( int xpos,int ypos,int width,int height, int waveOffset);
public:	
	SF_INFO* sfinfoin;	
	SF_INFO* sfinfoout;	
	static const int BAR_WIDTH = 30;
	static const int WAVE_HEIGHT = 60;
	static const int SEGMENT_HEIGHT = 12;

	//int sampleCount;// = 0;
	// wave correction rate
	double CORRECTION_RATE;// = .0003;
	WaveAnalysis* analyzeWave;
	// zoom factor
	float zoom;// = 512;

	// selection
	int selectionStart;// = -1;
	int selectionEnd;// = -1;
	// allow stopping of wave playback
	bool stopPlayingWave;// = false;

	WaveHandler() throw();
	void fileOpen();
	void fileSave();
	void fileSaveAs();

	// ensure file name ends with given extention
	string* addFileExtention(string* fileName, string* extention);
	
	// if returns false, user cancelled autosave action
	bool cancelledAutoSave();
	void startSelection( int at );	
	void extendSelection( int at );
	string* midiNoteName( int i );
	int midiToPos( double midi, int height );// convert midi pitch to screen position
	// FIXME: needs to add x offset
	WaveSegment* hitSegment( int x, int y,int xoff,int yoff, int height );
	
	bool isBlackKey( int midiNote );	
	void render(int x,int y,int width,int height, int waveOffset);
	void writeFile(string* fileName);
	void readWaveStream( fstream* file );
	void loadWave(string* fileName);
	void restoreOriginal();
	void resampleWaveIfRequired();
	void setVolume( float newVolume );
	void adjustVolume();
	void playWave(int startSample, int endSample );

/*	// play the sound on a background thread
	class PlayThread extends Thread {

		int startByte, endByte;
		byte[] samples;

		public PlayThread(int startAtByte, int endAtByte, byte samples[]) {
			this.startByte = startAtByte;
			this.endByte = endAtByte;
			this.samples = samples;
			stopPlayingWave = false;
		}

		public final void run() {

			// make sure a waveform is loaded
			if (byteCount == 0) {
				return;
			}

			// make sure audio device connected
			if (sourceDataLine == null) {
				Utilities.errMessage("Audio device failed. Try reloading file");
				return;
			}

			// is it open?
			if (!(sourceDataLine.isOpen())) {
				Utilities.errMessage("Data line is not open");
				sourceDataLine = null;
			}
			
			// adjust the dataline volume
			adjustVolume();
			
			// send out 1K at a time
			int bufferSize = (1024 * 4);						

			// calculate buffer
			int startOfBuffer = startByte;

			while (!WaveHandler.stopPlayingWave) {
				// check for buffer overflow
				if ((startOfBuffer + bufferSize) >= endByte) {
					bufferSize = (endByte) - startOfBuffer;
				}

				// start the line
				try {
					sourceDataLine.start();
				} catch (Exception e) {
					// oops
					System.out.println("Unable to start the line");
				}
				
				// fixme: throws illegal length if too large				
				int nBytesWritten = sourceDataLine.write(samples, startOfBuffer, bufferSize);

				// point to next starting point in the buffer
				startOfBuffer += bufferSize;
				if (startOfBuffer >= endByte) {
					break;
				}

			}
			
			// play out the remainder of the line, but don't close it
			sourceDataLine.stop();
			sourceDataLine.drain();
			sourceDataLine.flush();

		}

		
	}
	*/
};

#endif
