//MainWindow.java
//Main window for TunaFish pitch correction program
//
//(c) 2005 David Cuny
//http://tunafish.sourceforge.net
//Released under Qt Public License

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;


import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class MainWindow extends JFrame implements ActionListener {
	
	private static final long serialVersionUID = 1L;

	WavePanel wavePanel;			
	JScrollBar scrollBar;
	JSlider volumeSlider; 
	
	public MainWindow() {

		// set the name
		super("TunaFish Pitch Correction");
				
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// create the menu bar
		setJMenuBar(createMenuBar());

		// create the window content
		this.setContentPane(createContentPanel());
		
		// create the wave panel
		this.wavePanel = new WavePanel(200);
		this.getContentPane().add(this.wavePanel, BorderLayout.CENTER);
		
		// set the default volume
		volumeSlider.setValue(5);
	    //Main.mainWindow.wavePanel.wave.setVolume( 5 );
		
		// display the window.
		setSize(600, 500);
		setVisible(true);
		
	}
	

	public void setTitle( String fileName ) {
		super.setTitle( "TunaFish Pitch Correction - " + fileName );
	}
	
	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent arg0) {
		// TODO Auto-generated method stub
		
	}

		
	public JMenuBar createMenuBar() {
		JMenuBar menuBar;
		JMenu menu, subMenu;
		JMenuItem menuItem, subMenuItem;
		JRadioButtonMenuItem rbMenuItem;
		JCheckBoxMenuItem cbMenuItem;

		//Create the menu bar.
		menuBar = new JMenuBar();

		// file menu
		menu = new JMenu("File");
		menu.setMnemonic(KeyEvent.VK_F);
		menuBar.add(menu);

		
		//file : open
		menuItem = new JMenuItem("Open...", KeyEvent.VK_O);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
				ActionEvent.ALT_MASK));
		menuItem.getAccessibleContext().setAccessibleDescription(
				"Open a wave file");
		menu.add(menuItem);
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Main.mainWindow.wavePanel.wave.fileOpen();
			}
		});

		menu.addSeparator();

		//file : save
		menuItem = new JMenuItem("Save...", KeyEvent.VK_S);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
				ActionEvent.ALT_MASK));
		menuItem.getAccessibleContext().setAccessibleDescription(
				"Save the current wave file");
		menu.add(menuItem);
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Main.mainWindow.wavePanel.wave.fileSave(); 
			}
		});

		//file : save as
		menuItem = new JMenuItem("Save As...", KeyEvent.VK_A);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A,
				ActionEvent.ALT_MASK));
		menuItem.getAccessibleContext().setAccessibleDescription(
				"Save the current wave file using a different file name");
		menu.add(menuItem);
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Main.mainWindow.wavePanel.wave.fileSaveAs();				
			}
		});
		
		menu.addSeparator();
		
		// file exit
		menuItem = new JMenuItem("Exit", KeyEvent.VK_X);
		menuItem.getAccessibleContext().setAccessibleDescription(
				"Close the application");
		menuItem.addActionListener(this);
		menu.add(menuItem);
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// save current work?
				if (Main.mainWindow.wavePanel.wave.cancelledAutoSave()) {
					return;
				}
				// exit application
				System.exit(0);
			}
		});
		

		// help menu
		menu = new JMenu("Help");
		menu.setMnemonic(KeyEvent.VK_H);
		menuBar.add(menu);

		
		return menuBar;
	}
	
	
	public JPanel createContentPanel() {
		//Create the content-pane-to-be.
		JButton button;
		JPanel contentPanel = new JPanel(new BorderLayout());
		contentPanel.setPreferredSize(new Dimension(200, 200));
		contentPanel.setOpaque(true);

		// create a panel to put buttons in
		JToolBar toolbar = new JToolBar();
		toolbar.setPreferredSize(new Dimension(200, 40));
		contentPanel.add("North", toolbar);

		// file open
		button = new JButton(createImageIcon("images/open.gif"));
		button.setToolTipText("Load a file from disk");
		toolbar.add(button);
		button.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Main.mainWindow.wavePanel.wave.fileOpen();					
			}
		});

		// file save
		button = new JButton(createImageIcon("images/save.gif"));
		button.setToolTipText("Save a file to disk");
		toolbar.add(button);
		button.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Main.mainWindow.wavePanel.wave.fileSave();
			}
		});

		toolbar.addSeparator();
		
		// volume control
	    volumeSlider = new JSlider();
	    volumeSlider.setToolTipText("Adjust the playback volume");
	    volumeSlider.setMaximumSize( new Dimension(200, 40) );	    
	    volumeSlider.setMaximum( 10 );
	    volumeSlider.setMajorTickSpacing(10);
	    volumeSlider.setMinorTickSpacing(1);
	    volumeSlider.setPaintTicks(true);
	    volumeSlider.setPaintLabels(true);
	    toolbar.add(volumeSlider);	    
	    volumeSlider.addChangeListener( new ChangeListener () {
			public void stateChanged(ChangeEvent e) { 
				// set the volume in the waveHandler control
				float newVolume = volumeSlider.getValue();
				if (Main.mainWindow != null) {
					Main.mainWindow.wavePanel.wave.setVolume( newVolume );
				}
			}
	    });
		
		toolbar.addSeparator();
				
		// play
		button = new JButton(createImageIcon("images/play.gif"));
		button.setToolTipText("Play the wave");
		toolbar.add(button);
		button.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// number of frames
				//int end = Main.mainWindow.dopeSheet.waveFile.frameCount;
				// play the entire wave, return to current position
				Main.mainWindow.wavePanel.wave.playWave(-1,-1);
			}
		});

		// play selection
		button = new JButton(createImageIcon("images/play_selection.gif"));
		button.setToolTipText("Play the selected portion of the wave");
		toolbar.add(button);
		button.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// play from start of selection
				int selectionStart = Main.mainWindow.wavePanel.wave.selectionStart;
				int selectionEnd = Main.mainWindow.wavePanel.wave.selectionEnd;
				Main.mainWindow.wavePanel.wave.playWave(selectionStart,selectionEnd);
			}
		});

		// play to
		button = new JButton(createImageIcon("images/play_to.gif"));
		button.setToolTipText("Play the wave from the start to the current position");
		toolbar.add(button);
		button.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {				
				// play from start of selection
				int selectionEnd = Main.mainWindow.wavePanel.wave.selectionEnd;
				Main.mainWindow.wavePanel.wave.playWave(-1,selectionEnd);
			}
		});
		
		// play from
		button = new JButton(createImageIcon("images/play_from.gif"));
		button.setToolTipText("Play the wave from current position to the end");
		toolbar.add(button);
		button.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// play from start of selection
				int selectionStart = Main.mainWindow.wavePanel.wave.selectionStart;
				Main.mainWindow.wavePanel.wave.playWave(selectionStart,-1);
			}
		});


		// stop
		button = new JButton(createImageIcon("images/stock_media-stop.png"));
		button.setToolTipText("Stop playing the wave");
		toolbar.add(button);
		button.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				WaveHandler.stopPlayingWave = true;				
			}
		});
		
		toolbar.addSeparator();

		// zoom in on selection
		button = new JButton(createImageIcon("images/zoom_fit.gif"));
		button.setToolTipText("Zoom in on selection");
		toolbar.add(button);
		button.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// FIXME				
			}
		});


		// zoom normal
		button = new JButton(createImageIcon("images/zoom_1.gif"));
		button.setToolTipText("Normal view");
		toolbar.add(button);
		button.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// no zoom
				//Main.mainWindow.wavePanel.wave.zoom = 1;
				Main.mainWindow.wavePanel.adjustScrollbar();
				Main.mainWindow.wavePanel.waveCanvas.repaint();
			}
		});


		// zoom in
		button = new JButton(createImageIcon("images/zoom_in.gif"));
		button.setToolTipText("Zoom In");
		toolbar.add(button);
		button.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// smaller steps between
				//Main.mainWindow.wavePanel.wave.zoom /= 2;
				Main.mainWindow.wavePanel.adjustScrollbar();
				Main.mainWindow.wavePanel.waveCanvas.repaint();
			}
		});

		// zoom out
		button = new JButton(createImageIcon("images/zoom_out.gif"));
		button.setToolTipText("Zoom Out");
		toolbar.add(button);
		button.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// larger steps between
				//Main.mainWindow.wavePanel.wave.zoom *= 2;
				
				// adjust the scrollbar for the new zoom value
				Main.mainWindow.wavePanel.adjustScrollbar();
				
				// repaint the window
				Main.mainWindow.wavePanel.waveCanvas.repaint();
			}
		});
		
		toolbar.addSeparator();

		String[] keys = { "No Correction", "Chromatic", "A Major", "A# Major", "B Major", "C Major", "C# Major", "D Major", "D# Major", "E Major", "F Major", "F# Major", "G Major", "G# Major" }; 
		JComboBox combo = new JComboBox( keys );
		combo.setToolTipText("Select a Scale");
		toolbar.add(combo);
		combo.addActionListener(new java.awt.event.ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				// get the selection
				JComboBox cb = (JComboBox)e.getSource();
				String choice = (String)cb.getSelectedItem();
				
				if (choice.equalsIgnoreCase("no correction")) {
					// restore the original wave
					Main.mainWindow.wavePanel.wave.restoreOriginal();
				} else {					
					// set the scale
					Main.mainWindow.wavePanel.wave.analyzeWave.setScale( choice );
					
					// fix pitch relative to the scale
					Main.mainWindow.wavePanel.wave.analyzeWave.calculateCorrectPitch();
				}
								
				// repaint
				Main.mainWindow.wavePanel.repaint();
			}
		});

		// apply key signature
		button = new JButton(createImageIcon("images/stock_glue.png"));
		button.setToolTipText("Fix Notes");
		toolbar.add(button);
		button.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
			}
		});

		String[] methods = { "Cubic Spline", "Catmull-Clark Spline", "PSOLA" }; 
		combo = new JComboBox( methods );
		combo.setToolTipText("Select an Interpolation Method");
		toolbar.add(combo);
		combo.addActionListener(new java.awt.event.ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				// get the selection
				JComboBox cb = (JComboBox)e.getSource();
				String choice = (String)cb.getSelectedItem();
				
				if (choice.equals("Cubic Spline")) { 
					WaveBuilder.interpolation = WaveBuilder.USE_CUBIC_SPLINE;
					
				} else if (choice.equalsIgnoreCase("Catmull-Clark Spline")) { 
						WaveBuilder.interpolation = WaveBuilder.USE_CATMULL_CLARK;

				} else if (choice.equalsIgnoreCase("PSOLA")) { 
					WaveBuilder.interpolation = WaveBuilder.USE_PSOLA;
				}
								
				// force resynthesis
				Main.mainWindow.wavePanel.wave.analyzeWave.needsResampling = true;
			}
		});

		
		
		return contentPanel;
	}
		
	// return an icon
	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = MainWindow.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			String message = new String("Couldn't find file: " + path);
			JOptionPane.showMessageDialog(null, message, "Error",
					JOptionPane.ERROR_MESSAGE);
			return null;
		}
	}
		
}
