//WavePanel.java
//User interface to wave file
//
//(c) 2005 David Cuny
//http://tunafish.sourceforge.net
//Released under Qt Public License

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JToolBar;
import javax.swing.Scrollable;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.sun.java_cup.internal.emit;

/**
 * @author David Cuny
 * 
 */
public class WavePanel extends JPanel {

	private static final long serialVersionUID = 1L;
	
	public WaveHandler wave;
	public WaveCanvas waveCanvas;
	public JScrollBar horizontalScrollBar;
	public JScrollBar verticalScrollBar;
		
	public WavePanel(int pixels) {
		
		// create an empty wave file
		this.wave = new WaveHandler();
		
		this.setLayout(new BorderLayout() );
		this.setOpaque(true);
		
		// create a label to display the wave on
		this.waveCanvas = new WaveCanvas();
		this.add(this.waveCanvas, BorderLayout.CENTER);
		
		// create horizontal scroll bar to scroll the waveCanvas
		this.horizontalScrollBar = new JScrollBar(JScrollBar.HORIZONTAL);
		this.add(this.horizontalScrollBar, BorderLayout.SOUTH);
			
		this.horizontalScrollBar.addAdjustmentListener(new AdjustmentListener() {
			public void adjustmentValueChanged(AdjustmentEvent event) {
				Main.mainWindow.wavePanel.waveCanvas.repaint();
			}
		});

		
		// create vertical scroll bar to scroll the waveCanvas
		this.verticalScrollBar = new JScrollBar(JScrollBar.VERTICAL);
		this.add(this.verticalScrollBar, BorderLayout.EAST);
			
		this.verticalScrollBar.addAdjustmentListener(new AdjustmentListener() {
			public void adjustmentValueChanged(AdjustmentEvent event) {
				Main.mainWindow.wavePanel.waveCanvas.repaint();
			}
		});

		
	}		

	void adjustScrollbar() {
		// set maximum based on number of samples and zoom
		int max = (int)(Main.mainWindow.wavePanel.wave.sampleCount / Main.mainWindow.wavePanel.wave.zoom);		
		Main.mainWindow.wavePanel.horizontalScrollBar.setMaximum( max );
	}
	
	class WaveCanvas extends JPanel implements MouseListener, MouseMotionListener {

		// the current cursor
		int currentCursor = Cursor.DEFAULT_CURSOR;

		// position dragging began
		int mouseDragStartX;
		int mouseDragStartY;
		
		// midi value before drag
		int baseMidi = 0;
		
		// active segment (if any)
		WaveSegment selectedSegment = null;

		
		WaveCanvas() {
			// set up the mouse listener
			// event listeners
			this.addMouseListener(this);
			this.addMouseMotionListener(this);
			//this.addKeyListener(this);
		}
	 		
	    public void paintComponent(Graphics g) {
	    	
			// no wave loaded yet?
			if (Main.mainWindow.wavePanel == null) {
				return;
			}
	    	
			int points = 12;
						
			// repaint
			super.paintComponent(g);
			
			// get size
			int height = this.getHeight();
			int width = this.getWidth();
	
			// clear the background
			g.setColor(ColorScheme.NON_SCALE_BACKGROUND);			
			g.fillRect(0, 0, width, height);

			// get start position
			int position = Main.mainWindow.wavePanel.horizontalScrollBar.getValue(); 				

			// draw the wave on the panel
			Main.mainWindow.wavePanel.wave.render(g, height, width, position);
	    }
	    
		private final void useCursor( int id ) {

			// not the current cursor?
			if (this.currentCursor != id) {
				// make it current
				this.currentCursor = id;
				// use it
				this.setCursor( new Cursor(id) );
			}
		}

		
		void adjustCursor( int x, int y, boolean isShiftDown ) {
			// useCursor( Cursor.<TYPE> )
		}

		public void mouseMoved(MouseEvent e) {
			// adjust the cursor based on the state
			adjustCursor( e.getX(), e.getY(), e.isShiftDown() );		
		}	    

		public void mouseDragged(MouseEvent e) {
			// where is the click?
			int x = e.getX();
			int y = e.getY();
			
			// get offset
			int position = Main.mainWindow.wavePanel.horizontalScrollBar.getValue();
			
			// actual position
			float zoom = Main.mainWindow.wavePanel.wave.zoom;
			int at = (int)((position + x)*zoom);

			// is a segment being dragged?
			if (this.selectedSegment != null) {
				// calculate change in y position
				int dy = y - this.mouseDragStartY;

				// calculate relative change in terms of midi notes
				int addMidi = dy / WaveHandler.SEGMENT_HEIGHT;

				// moved less than a step?
				if (addMidi == 0) {
					// only change the position of the drag box
					this.selectedSegment.dragY = dy;
					
				} else {
					// reset the drag offset
					this.selectedSegment.dragY = 0;					
					this.mouseDragStartY = y;
				
					// add to user changed midi value
					this.selectedSegment.midiNumber -= addMidi;
					
					// resample the wave
					Main.mainWindow.wavePanel.wave.analyzeWave.calculateCorrectPitch();															
				}
				
				// repaint
				Main.mainWindow.wavePanel.repaint();
				
				return;
				
			}
			
			
			// no start selection?
			if (Main.mainWindow.wavePanel.wave.selectionStart == -1 ) {
				// start selection here
				Main.mainWindow.wavePanel.wave.selectionStart = at;				
			}
			
			// extend the selection
			Main.mainWindow.wavePanel.wave.extendSelection( at );
			
			// repaint
			Main.mainWindow.wavePanel.repaint();
		}

		
		public void mousePressed(MouseEvent e) {
			boolean repaintFlag = false;
			
			// get position
			int x = e.getX();
			int y = e.getY();
		
			// get the wave handler
			WaveHandler wave = Main.mainWindow.wavePanel.wave;
			
			// deactivate prior segment?
			if (this.selectedSegment != null) {
				// unselect the segment
				this.selectedSegment.isSelected = false;
				
				// force a repaint
				repaintFlag = true;
			}
			
			// hit a segment?
			this.selectedSegment = wave.hitSegment( x, y, this.getHeight() );
			if (this.selectedSegment != null) {				
				// make the selected segment active
				this.selectedSegment.isSelected = true;
			
				// save mouse position at the start of the drag
				this.mouseDragStartX = x;
				this.mouseDragStartY = y;
				
				// force a repaint
				repaintFlag = true;
				
			} else {
			
				// remove selection?
				if (Main.mainWindow.wavePanel.wave.selectionStart != -1 ) {				
					Main.mainWindow.wavePanel.wave.selectionStart = -1;
					repaintFlag = true;
				}
			}
			
			// repaint window?
			if (repaintFlag) {
				// update the window
				Main.mainWindow.wavePanel.repaint();
			}
		}

		public void mouseClicked(MouseEvent e) {
			// right click?
			if (e.getButton() == MouseEvent.BUTTON3) {
				// nothing
			}
			
			// doubleclick?
			if (e.getClickCount() == 2) {
				// nothing
			}
		}
		
		public void mouseReleased(MouseEvent e) {
			// check for end of drag
			if (this.selectedSegment != null ) {
				// clear the drag offset
				this.selectedSegment.dragY = 0;
				
				// repaint
				Main.mainWindow.wavePanel.repaint();
			}
		}

		public void mouseEntered(MouseEvent e) {
			// restore the default cursor
			this.currentCursor = Cursor.DEFAULT_CURSOR;
			this.setCursor( new Cursor(this.currentCursor) );
		}

		public void mouseExited(MouseEvent e) {
			// restore the default cursor?
			if (this.currentCursor != Cursor.DEFAULT_CURSOR) {
				this.currentCursor = Cursor.DEFAULT_CURSOR;
			}
		}
	}
}
