import java.awt.Color;
import java.awt.Graphics;

//WaveSegment.java
//Discrete chunk of a wave
//
//(c) 2005 David Cuny
//http://tunafish.sourceforge.net
//Released under Qt Public License

public class WaveSegment {
	boolean isSelected;
	int startSample;
	int endSample;	
	int midiNumber;
	int maxHeight;
	boolean overrideScale;
	boolean selected;
	int dragY;
	
	int x1;
	int y1;
	int x2;
	int y2;
	
	WaveSegment( int start, int end, int midi, int[] samples ) {
		this.startSample = start;
		this.endSample = end;		
		this.midiNumber = midi;
		this.selected = false;
		this.overrideScale = false;
		
		// find the maximum height
		this.maxHeight = 1;
		for (int i = startSample; i <= endSample; i++) {
			int sample = Math.abs(samples[i]);			
			if (sample > this.maxHeight) this.maxHeight = sample;
		}
	}
	
	
	public void calcScreenPosition( int xStart, int yStart, double zoom ) {
		this.x1 = xStart;
		this.y1 = yStart - (WaveHandler.SEGMENT_HEIGHT/2);
		this.x2 = x1 + (int)((this.endSample - this.startSample) / zoom);
		this.y2 = y1 + WaveHandler.SEGMENT_HEIGHT;
	}
	
	public void render(Graphics g, int xStart, int yStart, double zoom, int samples[]) {
		
		// calculate screen position
		this.calcScreenPosition( xStart, yStart, zoom);
					
		// selected?
		if (this.isSelected) {
			g.setColor(ColorScheme.SELECTED_SEGMENT);
		} else {
			g.setColor(ColorScheme.UNSELECTED_SEGMENT);
		}
		g.fillRect( this.x1, this.y1, this.x2 - this.x1, this.y2 - this.y1);
				
		// draw selection box?
		if (this.isSelected) {
			g.setColor(ColorScheme.SELECTED_SEGMENT_BORDER);
		} else {
			g.setColor(ColorScheme.UNSELECTED_SEGMENT_BORDER);
		}
		// draw selection box, offset by dragY value
		g.drawRect( this.x1, this.y1+this.dragY, this.x2 - this.x1, this.y2 - this.y1);

	}

	
}
